$(function() {
	var camera = {
		x: 0,
		y: 0,
		tx: 0,
		ty: 0,
		speed: 0.05,
		
		setPos: function(tx, ty) {
			this.tx = tx;
			this.ty = ty;
		},
		update: function() {
			this.x = lerp(this.x, this.tx, this.speed);
			this.y = lerp(this.y, this.ty, this.speed);
		}
	};
	
	var canvas = $('#canvas')[0];
	$(canvas).attr({
			'width': SCREEN_WIDTH,
			'height': SCREEN_HEIGHT
		}).css({
			'width': (SCREEN_WIDTH + "px"),
			'height': (SCREEN_HEIGHT + "px")
		});
	
	var ctx = canvas.getContext('2d');
	
	var plane;
	var enemy;
	
	var enemyTimer = new Timer(30);
	enemyTimer.onComplete = function() {
		enemyTimer.start();
		if (numEnemies < totalNumEnemies) {
			if (plane != undefined) {
				var furthestSpawn;
				var furthestSpawnDist = 0;
				for (var i = 0; i < spawnPoints.length; i++) {
					var spawn = spawnPoints[i];
					var spawnDist = plane.distanceTo(spawn);
					if (spawnDist > furthestSpawnDist) {
						furthestSpawn = spawn;
						furthestSpawnDist = spawnDist;
					}
				}
				
				var enemy = new EnemyPlane(furthestSpawn.x, furthestSpawn.y);
				enemy.liftOff(furthestSpawn.startSpeed);
				enemy.addOnKill(function() {
					numEnemies--;
					numKills++;
				});
				EntityManager.addEntity(enemy);
				numEnemies++;
			}
		}
	};
	var lootTimer = new Timer(320, 480);
	lootTimer.onComplete = function() {
		lootTimer.start();
		EntityManager.addEntity(new Loot());
	};
	var leftPlantTimer = new Timer(600);
	leftPlantTimer.onComplete = function() {
		var plant = new Plant(375, 414);
		plant.addOnKill(function() {
			leftPlantTimer.start();
		});
		EntityManager.addEntity(plant);
	};
	var rightPlantTimer = new Timer(600);
	rightPlantTimer.onComplete = function() {
		var plant = new Plant(SCREEN_WIDTH - 375, 414);
		plant.addOnKill(function() {
			rightPlantTimer.start();
		});
		EntityManager.addEntity(plant);
	};
	
	var Timers = {
		enemy: enemyTimer,
		loot: lootTimer,
		leftPlant: leftPlantTimer,
		rightPlant: rightPlantTimer,
		
		update: function() {
			this.enemy.update();
			this.loot.update();
			this.leftPlant.update();
			this.rightPlant.update();
		},
		start: function() {
			this.enemy.start();
			this.loot.start();
			this.leftPlant.start();
			this.rightPlant.start();
		}
	};
	
	// Wait for all resources to load
	$(Loader).ready(function() {
		start();
		
		mainLoop();
		setInterval(mainLoop, FRAME_WAIT);
	});
	
	var spawnPoints = [
		{x: 70, y: 545, startSpeed: 5},
		{x: SCREEN_WIDTH - 70, y: 545, startSpeed: 5},
		{x: 200, y: 120, startSpeed: 2},
		{x: SCREEN_WIDTH - 200, y: 120, startSpeed: 2}
	];
	
	var changedUp = false;
	var changedDown = false;
	var changedPlane = false;
	function preparation() {
		if (Input.up) {
			if (!changedUp) {
				changedUp = true;
				totalNumEnemies++;
				if (totalNumEnemies > 9) totalNumEnemies = 9;
			}
		} else {
			changedUp = false;
		}
		if (Input.down) {
			if (!changedDown) {
				changedDown = true;
				totalNumEnemies--;
				if (totalNumEnemies < 1) totalNumEnemies = 1;
			}
		} else {
			changedDown = false;
		}
		if (Input.tab) {
			if (!changedPlane) {
				changedPlane = true;
				if (!rightPlaneIsPlayer) {
					EntityManager.kill(enemy);
					enemy = new PlayerPlane(SCREEN_WIDTH - 70, 545);
					enemy.friendly = false;
					enemy.rotation = 180 + 40;
					enemy.img = Loader.paper_plane_enemy;
					Timers.enemy.stop();
					enemy.addOnKill(function() {
						if (State.state == STATE_PLAYING) {
							State.state = STATE_ENDED;
							State.winner = true;
						}
					});
				} else {
					EntityManager.kill(enemy);
					enemy = new EnemyPlane(SCREEN_WIDTH - 70, 545);
					numEnemies++;
					Timers.enemy.start();
					enemy.addOnKill(function() {
						numEnemies--;
						numKills++;
					});
				}
				EntityManager.addEntity(enemy);
				rightPlaneIsPlayer = !rightPlaneIsPlayer;
			}
		} else {
			changedPlane = false;
		}
		if (Input.space) {
			letsaGo();
		}
	};
	
	const STATE_PREPARATION = 0;
	const STATE_PLAYING = 1;
	const STATE_ENDED = 2;
	
	var State = {
		state: STATE_PREPARATION
	};
	var numKills;

	var rightPlaneIsPlayer = false;
	var totalNumEnemies = 1;
	var numEnemies;	
	function start() {
		numEnemies = 0;
		State.state = STATE_PREPARATION;
		Timers.start();
		
		var background = new Entity("Background", Loader.background, SCREEN_WIDTH / 2, SCREEN_HEIGHT / 2, SCREEN_WIDTH, SCREEN_HEIGHT);
		background.physical = false;
		EntityManager.addEntity(background);
		
		EntityManager.addEntity(new Furniture(Loader.table, 48, SCREEN_HEIGHT - 24, 128, 64));
		EntityManager.addEntity(new Furniture(Loader.table, SCREEN_WIDTH - 48, SCREEN_HEIGHT - 24, 128, 64));
		
		EntityManager.addEntity(new Furniture(Loader.window, 375, 400, 64, 64));
		EntityManager.addEntity(new Furniture(Loader.shelf, 200, 200, 128, 32));
		
		EntityManager.addEntity(new Furniture(Loader.window, SCREEN_WIDTH - 375, 400, 64, 64));
		EntityManager.addEntity(new Furniture(Loader.shelf, SCREEN_WIDTH - 200, 200, 128, 32));
		
		plane = new PlayerPlane(70, 545);
		EntityManager.addEntity(plane);
		plane.addOnKill(function() {
			if (State.state == STATE_PLAYING) {
				State.state = STATE_ENDED;
				State.winner = false;
			}
		});
		
		if (rightPlaneIsPlayer) {
			enemy = new PlayerPlane(SCREEN_WIDTH - 70, 545);
			enemy.friendly = false;
			enemy.rotation = 180 + 20;
			enemy.minInitialRotation = 180 + 40;
			enemy.maxInitialRotation = 180 + 5;
			enemy.img = Loader.paper_plane_enemy;
			Timers.enemy.stop();
			enemy.addOnKill(function() {
				if (State.state == STATE_PLAYING) {
					State.state = STATE_ENDED;
					State.winner = true;
				}
			});
		} else {
			enemy = new EnemyPlane(SCREEN_WIDTH - 70, 545);
			numEnemies++;
			Timers.enemy.start();
			enemy.addOnKill(function() {
				numEnemies--;
				numKills++;
			});
		}
		EntityManager.addEntity(enemy);
	}
	function restart() {
		EntityManager.clearEntities();
		start();
		
		letsaGo();
	}
	function letsaGo() {
		plane.liftOff(6);
		enemy.liftOff(rightPlaneIsPlayer ? 6.0 : 5);
		
		numKills = 0;
		
		State.state = STATE_PLAYING;
	}
	function mainLoop() {
		EntityManager.update();
		
		if (State.state == STATE_PREPARATION) {
			preparation();
		} else if (State.state == STATE_ENDED) {
			if (Input.space) {
				restart();
			} else if (Input.escape) {
				EntityManager.clearEntities();
				start();
				
				State.state = STATE_PREPARATION;
			}
		} else {
			Timers.update();
			
			if (Input.ctrl) {
				plane.shoot();
			}
			if (rightPlaneIsPlayer) {
				if (Input.enter) {
					enemy.shoot();
				}
			} else {
				if (Input.enter) {
					plane.shoot();
				}
			}
			
			var rotatedMainPlane = false;
			if (Input.a ^ Input.d) {
				if (Input.a) {
					plane.rotateLeft();
				} else {
					plane.rotateRight();
				}
				rotatedMainPlane = true;
			}
			if (rightPlaneIsPlayer) {
				if (Input.left ^ Input.right) {
					if (Input.left) {
						enemy.rotateLeft();
					} else {
						enemy.rotateRight();
					}
				}
			} else {
				if (!rotatedMainPlane) {
					if (Input.left ^ Input.right) {
						if (Input.left) {
							plane.rotateLeft();
						} else {
							plane.rotateRight();
						}
					}
				}
			}
		}

		GUI.update();
		
		ctx.save();
		
		EntityManager.render(ctx);
		
		GUI.render(ctx);
		
		if (State.state == STATE_PREPARATION) {
			ctx.fillStyle = "#FFF";
			ctx.font = "20px Arial";
			ctx.textAlign = "center";
			ctx.fillText("Press SPACE to start", SCREEN_WIDTH / 2, SCREEN_HEIGHT / 2);
			ctx.fillText("TAB to toggle CPU / Player", SCREEN_WIDTH / 2, SCREEN_HEIGHT / 2 + 220);
			
			ctx.font = "18px Arial";
			ctx.fillText("Player", 70, 545 - 50);
			var rightPlaneString = (rightPlaneIsPlayer ? "Player" : "CPU");
			ctx.fillText(rightPlaneString, SCREEN_WIDTH - 70, 545 - 50);
			ctx.font = "14px Arial";
			ctx.fillText("WASD + Ctrl", 70, 545 - 30);
			if (rightPlaneIsPlayer) {
				ctx.fillText("Arrow keys + Enter", SCREEN_WIDTH - 70, 545 - 30);
			} else {
				ctx.fillText("x" + totalNumEnemies, SCREEN_WIDTH - 70, 545 - 30);
				ctx.fillText("Up / down arrow", SCREEN_WIDTH - 70, 545 + 20);
				ctx.fillText("Increase / decrease", SCREEN_WIDTH - 70, 545 + 40);
			}
		} else if (State.state == STATE_ENDED) {
			ctx.fillStyle = "#FFF";
			ctx.font = "20px Arial";
			ctx.textAlign = "center";
			var winnerText = (rightPlaneIsPlayer ? ("Player " + (State.winner ? "one" : "two") + " is the winner!") : (State.winner ? "You won!" : "You lose"));
			ctx.fillText(winnerText, SCREEN_WIDTH / 2, SCREEN_HEIGHT / 2 - 40);
			ctx.fillText("Press SPACE to play again", SCREEN_WIDTH / 2, SCREEN_HEIGHT / 2);
			ctx.fillText("ESC to set up", SCREEN_WIDTH / 2, SCREEN_HEIGHT / 2 + 220);
		}
		if (State.state != STATE_PREPARATION && !rightPlaneIsPlayer) {
			ctx.fillStyle = "#FFF";
			ctx.font = "14px Arial";
			ctx.fillText(numKills, 32, 32);
		}
		
		ctx.restore();
	}
});